//Addon made by @GamerDos/@gamerdos on YouTube

import { Player, world, EntityHealthComponent, system, EntityInventoryComponent, EntityQueryOptions, ItemStack, MinecraftItemTypes, ItemTypes, CommandResult, MinecraftEffectTypes } from "@minecraft/server"
import { ActionFormData, ModalFormData } from "@minecraft/server-ui"
import { config, sendAllMessage, sendPlayerMessage, runCommands } from "./config"
import { commands } from "./lifesteal/handler"

let time: number = 0

system.runInterval(() => {
    time += 1
}, 20)

system.runInterval(() => {
    if (lifestealLogs.length > 500) {
        lifestealLogs.splice(0, lifestealLogs.length - 1)
    }
}, 1)

function random(length: number, autoCharacters: boolean, characters?: string) {
    if (autoCharacters == true) {
        const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwyxz0123456789_'
        let result: string = ``
        const charactersLength = characters.length
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength))
        }
        return result
    }
    if (autoCharacters == false) {
        let result: string = ``
        const charactersLength = characters.length
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength))
        }
        return result
    }
}

/**
 * @param playerName Player to get
 */
function getPlayer(playerName: string) {
    if (!playerName) return
    let players = Array.from(world.getAllPlayers())
    let target = players.filter(x => x.name.toLowerCase() == playerName.toLowerCase())
    return target[0]
}

const names: string[] = []

class Database {
    //This is iBlqzed Code, all credit goes to him

    protected readonly data: Map<string, any> = new Map()
    /**
     * The name of the database
     */
    readonly name: string
    /**
     * Create a new database!
     */
    constructor(name: string) {
        this.name = name.replace(/"/g, '\\"')
        if (names.includes(this.name)) throw new Error(`You can't have 2 of the same databases`)
        if (this.name.length > 13 || this.name.length === 0) throw new Error(`Database names can't be more than 13 characters long, and it can't be nothing!`)
        names.push(this.name)
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives add DB_${this.name} dummy`)
        try {
            world.scoreboard.getObjective(`DB_${this.name}`).getParticipants().forEach((e) => {
                const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'))
                this.data.set(dis[0], dis[1])
            })
        } catch { }
    }
    /*
     * The length of the database
     */
    get length(): number {
        return this.data.size
    }
    /**
     * Set a value from a key
     * @param {string} key Key to set
     * @param {any} value The value
     */
    set(key: string, value: any): void {
        if ((JSON.stringify(value).replace(/"/g, '\\"').length + key.replace(/"/g, '\\"').length + 3) >= 32767) throw new Error(`Database setter to long... somehow`)
        this.delete(key)
        world.getDimension(`overworld`).runCommandAsync(`scoreboard players set "[\\"${key.replace(/"/g, '\\"')}\\",${JSON.stringify(value).replace(/"/g, '\\"')}]" "DB_${this.name}" 0`)
        this.data.set(key, value)
    }
    /**
     * Get a value from a key
     * @param {string} key Key to get
     * @returns {any} The value that was set for the key (or undefined)
     */
    get(key: string): any {
        if (this.data.has(key)) return this.data.get(key)
    }
    /**
     * Test for whether or not the database has the key
     * @param {string} key Key to test for
     * @returns {boolean} Whether or not the database has the key
     */
    has(key: string): boolean {
        if (!this.data.has(key)) return false
        return true
    }
    /**
     * Delete a key from the database
     * @param {string} key Key to delete from the database
     */
    delete(key: string): void {
        if (!this.data.has(key)) return
        world.getDimension(`overworld`).runCommandAsync(`scoreboard players reset "[\\"${key.replace(/"/g, '\\"')}\\",${JSON.stringify(this.data.get(key)).replace(/"/g, '\\"')}]" "DB_${this.name}"`)
        this.data.delete(key)
    }
    /**
     * Get an array of all keys in the database
     * @returns {string[]} An array of all keys in the database
     */
    keys(): string[] {
        return Array.from(this.data.keys())
    }
    /**
     * Get an array of all values in the database
     * @returns {any[]} An array of all values in the database
     */
    values(): any[] {
        return Array.from(this.data.values())
    }
    /**
     * Clears all values in the database
     */
    clear(): void {
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives remove "DB_${this.name}"`);
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives add "DB_${this.name}" dummy`);
        this.data.clear()
    }
    /**
     * Loop through all keys and values of the database
     * @param {(key: string, value: any) => void} callback Code to run per loop
     */
    forEach(callback: (key: string, value: any) => void) {
        this.data.forEach((v, k) => callback(k, v))
    }
    *[Symbol.iterator](): IterableIterator<[string, any]> {
        yield* this.data.entries()
    }
}

let lifesteal = new Database(`lifesteal`)
if (!lifesteal.get(`maxhearts`)) {
    lifesteal.set(`maxhearts`, `20`)
}
if (!lifesteal.get(`envsteal`)) {
    lifesteal.set(`envsteal`, `true`)
}
if (!lifesteal.get(`banghosts`)) {
    lifesteal.set(`banghosts`, `false`)
}
// How many hearts you gain or lose when killing or dying
if (!lifesteal.get(`heartgl`)) {
    lifesteal.set(`heartgl`, `1`)
}
let revivelist = new Database(`revivelist`)
let deadplayers = new Database(`deadplayers`)

function getScore(player: Player, objective: string) {
    return world.scoreboard.getObjective(objective).getScore(player.scoreboard)
}

const lifestealLogs: string[] = []

/**
 * Made by WavePlayz#7915
 */
function sleep(seconds: number, ticks: boolean = true) {
    if (ticks == true) {
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable)
                else resolve()
            })
        })
    }
    if (ticks == false) {
        seconds = seconds * 20
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable)
                else resolve()
            })
        })
    }
}

// Hidden commands (fun commands and stuff, don't show up in ?help)
commands.push({
    name: `weee`,
    staffOnly: true,
    code: ({ player }) => {
        if (player.hasTag(`weee-cooldown`)) return

        player.applyKnockback(0, 0, 0, 5)
        player.runCommandAsync(`tag @s add weee-cooldown`)
        system.runTimeout(() => {
            player.runCommandAsync(`tag @s remove weee-cooldown`)
        }, 100)
    }
})

commands.push({
    name: `push`,
    staffOnly: true,
    code: ({ player, args }) => {
        let target = getPlayer(args[0])
        if (!target) return sendPlayerMessage(player, `§cFailed to find player`)
        if (!args[1]) return sendPlayerMessage(player, `§cArgument 2 needs to be a number`)
        if (isNaN(+args[1])) return sendPlayerMessage(player, `§cArgument 2 needs to be a number`)
        let viewDirection = player.getViewDirection()
        let strength = +args[1]

        target.applyKnockback(viewDirection.x, viewDirection.z, strength, strength / 5)
    }
})

commands.push({
    name: `teleport`,
    aliases: [`tp`],
    staffOnly: true,
    code: ({ player, args }) => {
        try {
            player.runCommandAsync(`teleport ${args.join(` `)}`)
        } catch (error) {
            sendPlayerMessage(player, `§c${error}`)
        }
    }
})




commands.push({
    name: `deadplayers`,
    code: ({ player, args }) => {
        sendPlayerMessage(player, `§e§lDead Players ---`)
        deadplayers.forEach(x => {
            sendPlayerMessage(player, `§6${x}`)
        })
    }
})

//Help command
commands.push({
    name: 'help',
    aliases: [
        'h'
    ],
    code: ({ player, args }) => {
        if (args[0] == 'withdraw') {

            withdrawHelp(player)
            return
        }
        if (args[0] == 'revive') {

            reviveHelp(player)
            return
        }
        if (args[0] == 'heart') {

            heartHelp(player)
            return
        }
        if (args[0] == 'welcome') {
            welcomeHelp(player)
            return
        }
        if (args[0] == 'adminpanel') {
            apHelp(player)
            return
        }

        sendPlayerMessage(player, `§e§lLifesteal Help Page --§r
§6Prefix: ${config.prefix}
Do ${config.prefix}help <command> for help on that command!

§eNormal Commands --
§6§lwithdraw <amount> §r§6| Withdraw one or more hearts from your heart count
§6§lrevive <player> §r§6| Revive a player (Must have a heart of life in hand)
§6§ldeadplayers §r§6| Show all dead player's names
§6§lheart <set/setmax/envsteal/etc> §r§6| All the heart commands in one
§6§lwelcome §r§6| Show the welcome GUI

§r§eGhost Commands --
§6§ljumpscare <player> §r§6| Jumpscare a player (must be near them)
§6§lrps <rock/paper/scissors> §r§6| Play rock paper scissors with an AI
§6§lcookieclicker §r§6| Play cookie clicker

§r§eStaff Only Commands --§r`)
        if (!player.hasTag(config.staffTag)) {
            if (player.isOp()) { }
            else return sendPlayerMessage(player, `§cYou do not have permission to see Staff Commands! Do /tag @s add lifesteal-opped to get permission.`)
        }
        sendPlayerMessage(player, `§6§ladminpanel §r§6| Open the admin panel
§6§llogs §r§6| Shows everytime a player used a command or gained a heart`)
    }
})

commands.push({
    name: `rps`,
    aliases: [`rockpaperscissors`],
    code: async ({ player, args }) => {
        if (!player.hasTag(`ls-dead`)) return sendPlayerMessage(player, `§cYou are not a ghost!`)
        let acceptable = [
            `rock`,
            `paper`,
            `scissors`
        ]
        if (!acceptable.includes(args[0])) return sendPlayerMessage(player, `§cYou can only do rock paper or scissors!`)
        let chosen1 = args[0]
        let chosen2 = random(1, false, `rps`)
        if (chosen2 == `r`) chosen2 = `rock`
        if (chosen2 == `p`) chosen2 = `paper`
        if (chosen2 == `s`) chosen2 = `scissors`
        sendPlayerMessage(player, `3`)
        player.playSound(`random.orb`)
        await sleep(10)
        sendPlayerMessage(player, `2`)
        player.playSound(`random.orb`, { pitch: 1.1 })
        await sleep(10)
        sendPlayerMessage(player, `1`)
        player.playSound(`random.orb`, { pitch: 1.2 })
        await sleep(10)
        player.playSound(`mob.villager.yes`)
        player.playSound(`mob.villager.no`)
        sendPlayerMessage(player, `<${player.name}> ${chosen1}`)
        sendPlayerMessage(player, `<AI> ${chosen2}`)
        await sleep(10)
        if (chosen1 == chosen2) return sendPlayerMessage(player, `§bTie!`)
        if (chosen1 == `rock`) {
            if (chosen2 == `paper`) {
                player.playSound(`mob.villager.no`)
                sendPlayerMessage(player, `§cYou lost!`)
                return
            }
            player.playSound(`mob.villager.yes`)
            sendPlayerMessage(player, `§aYou won!`)
        }
        if (chosen1 == `paper`) {
            if (chosen2 == `scissors`) {
                player.playSound(`mob.villager.no`)
                sendPlayerMessage(player, `§cYou lost!`)
                return
            }
            player.playSound(`mob.villager.yes`)
            sendPlayerMessage(player, `§aYou won!`)
        }
        if (chosen1 == `scissors`) {
            if (chosen2 == `rock`) {
                player.playSound(`mob.villager.no`)
                sendPlayerMessage(player, `§cYou lost!`)
                return
            }
            player.playSound(`mob.villager.yes`)
            sendPlayerMessage(player, `§aYou won!`)
        }
    }
})

commands.push({
    name: `guess`,
    code: ({ player, args }) => {
        if (!player.hasTag(`ls-dead`)) return sendPlayerMessage(player, `§cYou are not a ghost!`)
        if (isNaN(+args[0])) return sendPlayerMessage(player, `§cThats not a number!`)
        if (+args[0] <= 0 || +args[0] > 9) return sendPlayerMessage(player, `§cYou can only choose a number between 1-9!`)
        let chosen1 = args[0]
        let chosen2 = random(1, false, `123456789`)
        if (chosen1 == chosen2) {
            sendPlayerMessage(player, `§aYou won! It was ${chosen2}`)
            return
        }
        sendPlayerMessage(player, `§cYou lost! It was ${chosen2}`)
    }
})

commands.push({
    name: `cookieclicker`,
    aliases: [`cc`],
    code: ({ player, args }) => {
        if (!player.hasTag(`ls-dead`)) return sendPlayerMessage(player, `§cYou are not a ghost!`)
        let balance: number
        if (!balance) balance = 1
        sendPlayerMessage(player, `§3Close chat to open Cookie Clicker`)
        function don(player: Player) {
            const form = new ModalFormData()
                .title(`Cookie clicker`)
                .toggle(`Close\n§7Close game when you click submit`)
                .dropdown(`Click submit without any toggles on to get a cookie\nCookies:`, [`${balance}`])
            form.show(player).then(result => {
                if (result.canceled && result.cancelationReason == `userBusy`) don(player)
                if (result.formValues[0] == false) {
                    balance += 1
                    don(player)
                    return
                }
                if (result.formValues[0] == true) return
                don(player)
            })
        }
        don(player)
    }
})

commands.push({
    name: `logs`,
    staffOnly: true,
    code: ({ player, args }) => {
        sendPlayerMessage(player, `§6Lifesteal Logs:\n(The number before ">" is when it happened, resets to 0 when world restarts)\n${lifestealLogs.join(`\n`)}\n${time} > ${player.name} used - ?logs\n-----\nNote: Logs reset when the world restarts`)
    }
})

//Withdraw command
commands.push({
    name: 'withdraw',
    code: ({ player, args }) => {
        let amount = +args[0]
        if (isNaN(amount)) return sendPlayerMessage(player, `§cFailed to withdraw`)
        if (!amount) return withdrawHelp(player)
        if (amount > getScore(player, `ls-death`) || amount == getScore(player, `ls-death`)) return sendPlayerMessage(player, `§cYou cannot withdraw more hearts than you currently have!`)
        if (amount < 1) return sendPlayerMessage(player, `§cYou cannot withdraw less than 1 heart!`)
        if (amount > 64) return sendPlayerMessage(player, `§cYou cannot withdraw more than 64 hearts!`)

        player.runCommandAsync(`scoreboard players remove @s ls-death ${amount}`)
        player.runCommandAsync(`give @s lifesteal:heart ${amount}`)
        sendPlayerMessage(player, `§aWithdrew §f${amount}§a hearts`)
    }
})

//Revive command
commands.push({
    name: 'revive',
    code: ({ player, args }) => {
        let inv = player.getComponent(`inventory`) as EntityInventoryComponent
        let item = inv.container.getItem(player.selectedSlot)
        if (!item || item.typeId != `lifesteal:heart_of_life`) return sendPlayerMessage(player, `§cYou must have a heart of life to revive a player!`)

        if (!args[0]) return reviveHelp(player)
        let target
        deadplayers.forEach(x => {
            if (x.toLowerCase() == args.join(` `)) {
                target = x
            }
        })
        if (!target) return sendPlayerMessage(player, `§cFailed to find player, the player must be a ghost`)
        revivelist.set(`${target}`, `${target}`)

        player.runCommandAsync(`clear @s lifesteal:heart_of_life 0 1`)
        sendAllMessage(player, `§b${player.name} has revived ${target}`)

        player.runCommandAsync(`playsound heart_of_life_sound @a ~ ~ ~ 999`)
    }
})

//Welcome command
commands.push({
    name: 'welcome',
    code: ({ player, args }) => {
        if (!player.hasTag(`welcome-gui_done`)) return sendPlayerMessage(player, `§3Close chat to open GUI`)

        player.removeTag(`welcome-gui_done`)
        sendPlayerMessage(player, `§3Close chat to open GUI`)
    }
})

//Heart commands
commands.push({
    name: 'heart',
    code: ({ player, args }) => {

        if (!args[0]) return heartHelp(player)
        switch (args[0]) {
            default:
                heartHelp(player)
                break;
            case `banghosts`:
                if (!player.hasTag(config.staffTag)) {
                    if (player.isOp()) { }
                    else return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
                }
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid argument(s) detected, must be true or false`)
                        break;
                    case `true`:
                        lifesteal.set(`banghosts`, `true`)
                        sendPlayerMessage(player, `§aSet §fbanGhosts§a to §ftrue`)
                        break;
                    case `false`:
                        lifesteal.set(`banghosts`, `false`)
                        sendPlayerMessage(player, `§aSet §fbanGhosts§a to §ffalse`)
                        break;
                }
                break;
            case `heartgl`:
                if (!player.hasTag(config.staffTag)) {
                    if (player.isOp()) { }
                    else return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
                }
                if (isNaN(+args[1])) return sendPlayerMessage(player, `§cThats not a number!`)
                if (+args[1] == 0 || +args[1] > 4) return sendPlayerMessage(player, `§cYou can only set HeartGL to 1-4`)
                lifesteal.set(`heartgl`, `${args[1]}`)
                sendPlayerMessage(player, `§aSet §fheartGL §ato §f${args[1]}`)
                break;
            case `display`:
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid arguments detected, do ${config.prefix}help heart for a list of arguments`)
                        break;
                    case `on`:
                        if (player.hasTag(`heart-display_on`)) return sendPlayerMessage(player, `§cHeart display (public) is already on!`)
                        player.addTag(`heart-display_on`)
                        sendPlayerMessage(player, `§6Heart display (public) turned on (Might be glitchy)`)
                        break;
                    case `off`:
                        if (!player.hasTag(`heart-display_on`)) return sendPlayerMessage(player, `§cHeart display (public) is already off!`)
                        player.removeTag(`heart-display_on`)
                        sendPlayerMessage(player, `§6Heart display (public) turned off`)
                        break;
                }
                break;
            case `show`:
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid arguments detected, do ${config.prefix}help heart for a list of arguments`)
                        break;
                    case 'on':
                        if (player.hasTag(`heart-show_on`)) return sendPlayerMessage(player, `§cHeart display (private) is already on!`)
                        player.addTag(`heart-show_on`)
                        sendPlayerMessage(player, `§6Heart display (private) turned on (Might be glitchy)`)
                        break;
                    case 'off':
                        if (!player.hasTag(`heart-show_on`)) return sendPlayerMessage(player, `§cHeart display (private) is already off!`)
                        player.removeTag(`heart-show_on`)
                        sendPlayerMessage(player, `§6Heart display (private) turned off`)
                        break;
                }
                break;
            case 'set':
                if (!player.hasTag(config.staffTag)) {
                    if (player.isOp()) { }
                    else return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
                }
                let amount = args[1]
                if (!amount || isNaN(+amount)) return sendPlayerMessage(player, `§cInvalid amount entered`)
                if (!args[2]) {
                    if (!amount) return sendPlayerMessage(player, `§cInvalid amount entered`)
                    if (player.hasTag(`ls-dead`)) {
                        player.runCommandAsync(`tag @s remove ls-dead`)
                        player.runCommandAsync(`scoreboard players set @s ls-death ${amount}`)
                        player.runCommandAsync(`gamemode s`)
                    }
                    player.runCommandAsync(`scoreboard players set @s ls-death ${amount}`)
                    sendPlayerMessage(player, `§aSuccessfully set §f${player.name}'s§a hearts to §f${amount}`)
                    return
                }

                let target = getPlayer(args.slice(2, args.length).join(` `))
                if (!target) return sendPlayerMessage(player, `§cFailed to find target`)

                if (target.hasTag(`ls-dead`)) {
                    target.runCommandAsync(`tag @s remove ls-dead`)
                    target.runCommandAsync(`scoreboard players set @s ls-death ${amount}`)
                    target.runCommandAsync(`gamemode s`)
                }
                target.runCommandAsync(`scoreboard players set @s ls-death ${amount}`)
                sendPlayerMessage(player, `§aSuccessfully set §f${target.name}'s§a hearts to §f${amount}`)
                break;
            case 'setmax':
                if (!player.hasTag(config.staffTag)) {
                    if (player.isOp()) { }
                    else return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
                }
                if (!args[1] || isNaN(+args[1])) return sendPlayerMessage(player, `§cThat isn't a number!`)
                if (+args[1] > 100) return sendPlayerMessage(player, `§cYou can't set the max hearts to over 100!`)
                if (+args[1] < 10) return sendPlayerMessage(player, `§cYou can't set the max hearts to less than 10!`)
                lifesteal.set(`maxhearts`, `${args[1]}`)
                sendPlayerMessage(player, `§aSet maximum hearts to §f${args[1]}`)
                break;
            case 'envsteal':
                if (!player.hasTag(config.staffTag)) {
                    if (player.isOp()) { }
                    else return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
                }
                let acceptable = [`true`, `false`]
                if (!args[1] || !acceptable.includes(args[1].toLowerCase())) return sendPlayerMessage(player, `§cArgument must be true or false`)
                lifesteal.set(`envsteal`, `${args[1]}`)
                sendPlayerMessage(player, `§aSet environment steal to §f${args[1]}`)
                break;
        }
    }
})

//Ghost jumpscare
commands.push({
    name: 'jumpscare',
    code: ({ player, args }) => {
        if (!player.hasTag(`ls-dead`)) return sendPlayerMessage(player, `§cYou are not a ghost!`)
        if (player.hasTag(`ls-ghost_jumpscare_cd`)) return sendPlayerMessage(player, `§cYou are on cooldown! Wait ${Math.round(getScore(player, `ls-ghost-cd`) / 20)} more seconds!`)

        let target = getPlayer(args.join(` `))

        if (!target) return sendPlayerMessage(player, `§cFailed to run command, player must be online and atleast 5 blocks near you`)
        if (target.name == player.name) return sendPlayerMessage(player, `§cYou cannot jumpscare yourself!`)

        try {
            player.runCommandAsync(`title @a[name="${target.name}",r=5] actionbar §cA ghost has jumpscared you!`)
            target.runCommandAsync(`playsound mob.enderdragon.death @s ~ ~ ~ 1 0.7`)
            target.runCommandAsync(`playsound mob.wither.death @s ~ ~ ~ 1 0.7`)
            target.runCommandAsync(`playsound mob.wither.death @s ~ ~ ~ 1 1.3`)
            target.runCommandAsync(`playsound ambient.weather.thunder @s ~ ~ ~ 1 3`)
            player.runCommandAsync(`title ${target.name} title §c§lBOO`)
            player.runCommandAsync(`title ${target.name} subtitle §c${player.name} has jumpscared you!`)
            player.runCommandAsync(`scoreboard players set @s ls-ghost-cd 6002`)
            player.runCommandAsync(`tag @s add ls-ghost_jumpscare_cd`)
        }
        catch {
            sendPlayerMessage(player, `§cFailed to run command, player must be online and atleast 5 blocks near you`)
        }
    }
})

//Admin panel command
commands.push({
    name: 'adminpanel',
    aliases: [
        'ap',
        'ag',
        'admingui'
    ],
    staffOnly: true,
    code: data => {
        let player = data.player

        if (player.hasTag(`admin_gui`)) return sendPlayerMessage(player, `§3Close chat to open GUI`)

        sendPlayerMessage(player, `§3Close chat to open GUI`)
        function form(player: Player) {
            const agMain = new ActionFormData()
                .title(`Admin Panel`)
                .body(`Choose a command!`)
                .button(`Heart Commands`, `textures/items/ls-heart.png`)
                .button(`Utility Commands`, `textures/ui/Wrenches1`)
                .button(`Close`, `textures/ui/realms_red_x.png`)
            agMain.show(player).then(result => {
                if (result.canceled && result.cancelationReason == `userBusy`) form(player)
                if (result.selection == 0) {
                    const agheart = new ActionFormData()
                        .title(`Admin Panel - Heart`)
                        .body(`Choose a command!`)
                        .button(`Set hearts`, `textures/items/heart_essence.png`)
                        .button(`Reset hearts`, `textures/items/heart_fragment.png`)
                        .button(`Set maximum hearts`, `textures/items/ls-heart.png`)
                        .button(`Kill or revive`, `textures/items/heart_of_life.png`)
                        .button(`Set amount of hearts gained/lost`, `textures/items/withdraw_heart.png`)
                        .button(`Option to ban ghosts`, `textures/items/diamond_axe.png`)
                        .button(`Close`, `textures/ui/realms_red_x.png`)
                    agheart.show(player).then(result => {
                        switch (result.selection) {
                            case 0:

                                const playerArray: Player[] = []
                                const options: string[] = []

                                for (let players of world.getAllPlayers()) {
                                    playerArray.push(players)
                                    options.push(players.name)
                                }

                                const agheart_set = new ModalFormData()
                                    .title(`Set a player's hearts`)
                                    .dropdown(`Choose a player`, options)
                                    .slider(`Choose a heart count to set them to`, 0, 100, 1)
                                agheart_set.show(player).then(result => {
                                    let formValues = result.formValues
                                    let target = playerArray[formValues[0]]
                                    let score = getScore(target, `ls-death`)
                                    target.runCommandAsync(`scoreboard players set @s ls-death ${formValues[1]}`)
                                    sendPlayerMessage(player, `§aSet §f${target.name}'s§a hearts to §f${formValues[1]}.`)
                                    if (score == 0) {
                                        runCommands(target, [
                                            `scoreboard players set @s ls-death ${formValues[1]}`,
                                            `tag @s remove ls-dead`,
                                            `gamemode s @s[m=!s]`,
                                            `effect @s resistance 5 255 true`
                                        ])
                                    }
                                })
                                break;
                            case 1:

                                const playerArray2: Player[] = []
                                const options2: string[] = []

                                for (let players of world.getAllPlayers()) {
                                    playerArray2.push(players)
                                    options2.push(players.name)
                                }

                                const agheart_reset = new ModalFormData()
                                    .title(`Reset a player's hearts`)
                                    .dropdown(`Choose a player`, options2)
                                agheart_reset.show(player).then(result => {
                                    const formValues = result.formValues
                                    let target = playerArray2[formValues[0]]
                                    let score = getScore(target, `ls-death`)
                                    target.runCommandAsync(`scoreboard players set @s ls-death 10`)
                                    sendPlayerMessage(player, `§aReset §f${target.name}'s§a hearts to §f10.`)
                                    if (score == 0) {
                                        runCommands(target, [
                                            `scoreboard players set @s ls-death 10`,
                                            `tag @s remove ls-dead`,
                                            `gamemode s @s[m=!s]`,
                                            `effect @s resistance 5 255 true`
                                        ])
                                    }
                                })
                                break;
                            case 2:
                                const agheart_setmax = new ModalFormData()
                                    .title(`Set the maximum amount of hearts`)
                                    .slider(`Choose an amount of hearts to set the max to`, 10, 100, 1)
                                agheart_setmax.show(player).then(result => {
                                    const formValues = result.formValues
                                    lifesteal.set(`maxhearts`, `${formValues[0]}`)
                                    sendPlayerMessage(player, `§aSet the max hearts to §f${formValues[0]}.`)
                                })
                                break;
                            case 3:
                                let agheart_kr = new ActionFormData()
                                    .title(`Kill or revive`)
                                    .button(`Revive a player`, `textures/items/heart_of_life.png`)
                                    .button(`Kill a player\n(Set their hearts to 0)`, `textures/items/iron_sword.png`)
                                    .button(`Close`, `textures/ui/realms_red_x.png`)
                                agheart_kr.show(player).then(result => {
                                    switch (result.selection) {
                                        case 0:

                                            const options: string[] = []

                                            deadplayers.forEach(x => {
                                                options.push(x)
                                            })

                                            if (!options.length) {
                                                const errorGui = new ModalFormData()
                                                    .title(`Revive GUI`)
                                                    .dropdown(`Choose a player to revive`, [`No dead players!`])
                                                errorGui.show(player)
                                                return
                                            }

                                            const reviveGUI = new ModalFormData()
                                                .title(`Revive GUI`)
                                                .dropdown(`Choose a player to revive`, options)
                                            reviveGUI.show(player).then(result => {
                                                let formValues = result.formValues
                                                let target = options[formValues[0]]

                                                revivelist.set(`${target}`, `${target}`)
                                                deadplayers.delete(`${target}`)
                                                player.runCommandAsync(`playsound heart_of_life_sound @a ~ ~ ~ 999`)
                                                sendAllMessage(player, `§b${player.name} has revived ${target}`)
                                            })
                                            break;
                                        case 1:
                                            const playerArray3: Player[] = []
                                            const options3: string[] = []

                                            for (let players of world.getAllPlayers()) {
                                                playerArray3.push(players)
                                                options3.push(players.name)
                                            }
                                            const agheart_kr = new ModalFormData()
                                                .title(`Kill a player`)
                                                .dropdown(`Choose a player`, options3)
                                            agheart_kr.show(player).then(result => {
                                                let formValues = result.formValues
                                                let target = playerArray3[formValues[0]]
                                                if (target.hasTag(`ls-dead`)) return sendPlayerMessage(player, `§cThat player is already dead!`)
                                                target.runCommandAsync(`scoreboard players set @s ls-death 0`)
                                                sendPlayerMessage(player, `§aSet §f${target.name}'s§a hearts to §f0.`)
                                            })
                                            break;
                                    }
                                })
                                break;
                            case 4:
                                let heartglGui = new ModalFormData()
                                    .title(`HeartGL`)
                                    .slider(`Set the amount of hearts you gain when you kill a player, and hearts you lose when you die (RECOMMENDED NOT TO BE CHANGED)`, 1, 4, 1)
                                heartglGui.show(player).then(result => {
                                    lifesteal.set(`heartgl`, `${result.formValues[0]}`)

                                    sendPlayerMessage(player, `§aSet §fHeartGL§a to §f${result.formValues[0]}`)
                                })
                                break;
                            case 5:
                                let banGhostsGui = new ModalFormData()
                                    .title(`Ban Ghosts`)
                                    .toggle(`If this toggle is on, losing all hearts will ban you, instead of becoming a ghost (Not recommended for worlds, might make the dead player quit)`, JSON.parse(lifesteal.get(`banghosts`)))
                                banGhostsGui.show(player).then(result => {
                                    lifesteal.set(`banghosts`, `${result.formValues[0]}`)

                                    if (result.formValues[0] == true) sendPlayerMessage(player, `§aTurned §fbanGhosts on`)
                                    if (result.formValues[0] == false) sendPlayerMessage(player, `§aTurned §fbanGhosts off`)
                                })
                                break;
                        }
                    })
                }
                if (result.selection == 1) {
                    const agutil = new ActionFormData()
                        .title(`Admin Panel - Utility`)
                        .button(`Teleport`, `textures/ui/portalBg.png`)
                        .button(`Close`, `textures/ui/realms_red_x.png`)
                    agutil.show(player).then(result => {
                        if (result.selection == 0) {
                            const agutil_tp = new ActionFormData()
                                .title(`Utility - Teleport`)
                                .button(`Teleport to a player`, `textures/ui/FriendsIcon.png`)
                                .button(`Teleport to coordinates`, `textures/ui/xyz_axis.png`)
                                .button(`Teleport between dimensions`, `textures/ui/portalBg.png`)
                                .button(`Close`, `textures/ui/realms_red_x.png`)
                            agutil_tp.show(player).then(result => {
                                if (result.selection == 0) {
                                    const playerArray: Player[] = []
                                    const options: string[] = []

                                    for (let players of world.getAllPlayers()) {
                                        playerArray.push(players)
                                        options.push(players.name)
                                    }

                                    const tp_player = new ModalFormData()
                                        .title(`Teleport a player`)
                                        .dropdown(`Choose a player`, options)
                                        .toggle(`Teleport them to you`, false)
                                    tp_player.show(player).then(result => {
                                        const formValues = result.formValues
                                        let target = playerArray[formValues[0]]
                                        if (formValues[1] == false) {
                                            let location = target.location
                                            player.teleport(location, target.dimension, player.getRotation().x, player.getRotation().y)
                                            sendPlayerMessage(player, `§aTeleported you to §f${target.name}.`)
                                        }
                                        if (formValues[1] == true) {
                                            let location = player.location
                                            target.teleport(location, player.dimension, player.getRotation().x, player.getRotation().y)
                                            sendPlayerMessage(player, `§aTeleported §f${target.name}§a to you. `)
                                        }
                                    })
                                }
                                if (result.selection == 1) {
                                    const tp_coords = new ModalFormData()
                                        .title(`Teleport to coordinates`)
                                        .textField(`Type Coordinates`, `Enter coordinates`)
                                    tp_coords.show(player).then(result => {
                                        try {
                                            player.runCommandAsync(`teleport @s ${result.formValues[0]}`)
                                            sendPlayerMessage(player, `§aSuccessfully teleport you to §f${result.formValues[0]}.`)
                                        }
                                        catch {
                                            sendPlayerMessage(player, `§cFailed to teleport you to ${result.formValues[0]}`)
                                        }
                                    })
                                }
                                if (result.selection == 2) {
                                    let teleportDimensionGui = new ActionFormData()
                                        .title(`Teleport dimensions`)
                                        .body(`§gYou will teleport to the corresponding coordinates, e.g if you are in the overworld and you travel to the nether, you will be at your coordinates divided by 8`)
                                        .button(`The Overworld`)
                                        .button(`The Nether`)
                                        .button(`The End`)
                                        .button(`Close`)
                                    teleportDimensionGui.show(player).then(result => {
                                        switch (result.selection) {
                                            case 0:
                                                if (player.dimension.id == `overworld`) return sendPlayerMessage(player, `§cYou are already in the Overworld!`)
                                                if (player.dimension.id == `nether`) player.teleport({ x: player.location.x * 8, y: player.location.y * 8, z: player.location.z * 8 }, world.getDimension(`overworld`), player.getRotation().x, player.getRotation().y)

                                                player.teleport({ x: player.location.x, y: player.location.y, z: player.location.z }, world.getDimension(`overworld`), player.getRotation().x, player.getRotation().y)
                                                break;
                                            case 1:
                                                if (player.dimension.id == `nether`) return sendPlayerMessage(player, `§cYou are already in the Nether!`)
                                                if (player.dimension.id == `overworld`) return player.teleport({ x: player.location.x / 8, y: player.location.y / 8, z: player.location.z / 8 }, world.getDimension(`nether`), player.getRotation().x, player.getRotation().y)

                                                player.teleport({ x: player.location.x / 8, y: player.location.y / 8, z: player.location.z / 8 }, world.getDimension(`nether`), player.getRotation().x, player.getRotation().y)
                                                break;
                                            case 2:
                                                if (player.dimension.id == `the_end`) return sendPlayerMessage(player, `§cYou are already in the End!`)

                                                player.teleport({ x: player.location.x, y: player.location.y, z: player.location.z }, world.getDimension(`end`), player.getRotation().x, player.getRotation().y)
                                                break;
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        }
        form(player)
    }
})


//Help for commands
function withdrawHelp(player: Player) {
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Withdraw§r§6
> Withdraw a certain amount of hearts into your inventory

> Usage: ${config.prefix}withdraw <amount>

> Examples: [
${config.prefix}withdraw 3
${config.prefix}withdraw 6
]`)
}


function reviveHelp(player: Player) {
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Revive§r§6
> Revive a player if they are a ghost, requires a heart of life in hand to work

> Usage: ${config.prefix}revive <player>

> Examples: [
${config.prefix}revive ${player.name}
${config.prefix}revive TestSubject8987
]`)
}


function heartHelp(player: Player) {
    if (player.hasTag(config.staffTag)) {
        sendPlayerMessage(player, `§e§lLifesteal Help Page -- Heart§r§6
> Withdraw a certain amount of hearts into your inventory

> Usage: ${config.prefix}heart <display/show/set/reset/setmax>

> Args: [
    display - Display your max health above your head
    show - Display your max health to only yourself
    set - Set a player's max health
    setmax - Set the maximum amount of hearts
    envsteal - Toggle environment steal (whether you lose hearts from natural causes like mobs)
    banghosts - Option to ban ghosts
    heartgl - Set the amount of hearts you gain or lose when you kill a player or die 
]

> Examples: [
    ${config.prefix}heart display on
    ${config.prefix}heart display off
    ${config.prefix}heart show on
    ${config.prefix}heart show off
    ${config.prefix}heart set 16 ${player.name}
    ${config.prefix}heart set 16
    ${config.prefix}heart setmax 30
    ${config.prefix}heart envsteal true
    ${config.prefix}heart envsteal false
    ${config.prefix}heart banghosts true
    ${config.prefix}heart banghosts false
    ${config.prefix}heart heartgl 4
]`)
        return
    }
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Heart§r§6
> Withdraw a certain amount of hearts into your inventory

> Usage: ${config.prefix}heart <display/show/set/setmax>

> Args: [
display - Display your max health above your head
show - Display your max health to only yourself
]

> Examples: [
${config.prefix}heart display on
${config.prefix}heart display off
${config.prefix}heart show on
${config.prefix}heart show off
]`)
}


function welcomeHelp(player: Player) {
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Welcome§r§6
> Show the welcome GUI

> Usage: ${config.prefix}welcome

> Examples: [
${config.prefix}welcome
]`)
}

function apHelp(player: Player) {
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Welcome§r§6
> Show the welcome GUI

> Usage: ${config.prefix}adminpanel

> Aliases: [
    ap
    admingui
    ag
]

> Examples: [
    ${config.prefix}adminpanel
]`)
}


world.events.beforeChat.subscribe(data => {
    if (data.message.startsWith(config.prefix)) {
        data.cancel = true
        const args = data.message.slice(config.prefix.length).toLowerCase().split(/\s+/g)
        const command = args.shift()
        const cmdData = commands.find(c => c.name.toLowerCase() === command.toLowerCase() || c.aliases?.includes(command.toLowerCase()))
        if (!cmdData) return sendPlayerMessage(data.sender, `§cUnknown command: ${data.message.slice(config.prefix.length)}. Do ${config.prefix}help for a list of commands`)
        if (cmdData.staffOnly && !data.sender.hasTag(config.staffTag)) {
            if (data.sender.isOp()) { }
            else return sendPlayerMessage(data.sender, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`)
        }

        if (cmdData.gamerdosOnly && !data.sender.name.includes(`GamerDos7511`)) return sendPlayerMessage(data.sender, `§cUnknown command: ${data.message.slice(config.prefix.length)}. Do ${config.prefix}help for a list of commands`)

        let playerTags = data.sender.getTags()
        if (playerTags) {
            if (cmdData.gamerdosOnly && !playerTags.filter(x => x == `GamerDos`)) return sendPlayerMessage(data.sender, `§cUnknown command: ${data.message.slice(config.prefix.length)}. Do ${config.prefix}help for a list of commands`)
        }

        cmdData.code({ player: data.sender, args: args })
        if (cmdData.gamerdosOnly == true) return
        lifestealLogs.push(`§6${time} > ${data.sender.name} used - ${data.message}`)
    }
})

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let target
        if (player.hasTag(`ls-revive`)) {
            target = player
        }
        if (!target) return
        if (!target.hasTag(`ls-dead`)) return target.removeTag(`ls-revive`)
        target.runCommandAsync(`scoreboard players set @s ls-death 5`)
        target.runCommandAsync(`tag @s remove ls-dead`)
        runCommands(target, [
            `gamemode s @s[m=!s]`,
            `effect @s resistance 5 255 true`
        ])

        target.teleport(target.getSpawnPosition(), target.spawnDimension, target.getRotation().x, target.getRotation().y)
    }
}, 1)

system.runInterval(async () => {
    for (let player of world.getAllPlayers()) {
        let health = player.getComponent(`health`) as EntityHealthComponent
        if (player.hasTag(`heart-show_on`)) {
            player.runCommandAsync(`title @s actionbar §a${Math.round(health.current / 2)}/${getScore(player, `ls-death`)}`)
        }
        if (player.hasTag(`ls-dead`)) {
            let entites = [...world.getDimension(`overworld`).getEntities({ location: { x: 0, y: 90, z: 0 }, minDistance: 100 })]
            if (entites.find((x: Player) => x.name == player.name)) {
                if (player.hasTag(`ls-revive`)) return
                player.teleport({ x: 0, y: 90, z: 0 }, player.dimension, player.getRotation().x, player.getRotation().y)
                sendPlayerMessage(player, `§cYou may not leave spawn!`)
                await sleep(1, true)
                player.runCommandAsync(`playsound note.pling @s ~ ~ ~ 1 1.1`)
            }

        }
        if (getScore(player, `ls-death`) > +lifesteal.get(`maxhearts`)) {
            let current = getScore(player, `ls-death`)
            player.runCommandAsync(`scoreboard players set @s ls-death ${lifesteal.get(`maxhearts`)}`)
            let amount = current - +lifesteal.get(`maxhearts`)
            player.runCommandAsync(`give @s lifesteal:heart ${amount}`)
        }
        if (system.currentTick % 20 !== 0) return
        if (player.hasTag(`heart-display_on`)) {
            player.nameTag = `${player.name}\n§c${Math.round(health.current / 2)}/${getScore(player, `ls-death`)}`
        }
    }
}, 1)

world.events.playerSpawn.subscribe(({ player, initialSpawn }) => {
    if (initialSpawn == false) return
    sendPlayerMessage(player, `§c-----
§6Lifesteal version: v1.12.3-release
§1§lGamerDos§r§1 on §cYou§fTube§1.
§c-----`)

    if (player.hasTag(`ls-player`)) return
    player.runCommandAsync(`spawnpoint @s`)
})

world.events.playerSpawn.subscribe(({ player, initialSpawn }) => {
    if (initialSpawn == false) return
    if (player.name !== `GamerDos7511`) return
    sendAllMessage(player, `§c§lThe creator of this Lifesteal Addon/Plugin has joined the game! (${player.name})`)
    player.runCommandAsync(`playsound ambient.weather.thunder @a`)
    if (!player.hasTag(`lifesteal-opped`)) player.addTag(`lifesteal-opped`)
})

world.events.beforeItemUse.subscribe(data => {
    let player = data.source as Player
    let item = data.item
    if (item.typeId == `lifesteal:heart`) {
        if (getScore(player, `ls-death`) >= 100) {
            player.runCommandAsync(`playsound note.bass @s`)
            return sendPlayerMessage(player, `§cYou are at the max amount of hearts!`)
        }
        if (getScore(player, `ls-death`) >= +lifesteal.get(`maxhearts`)) {
            player.runCommandAsync(`playsound note.bass @s`)
            data.cancel = true
            return sendPlayerMessage(player, `§cYou are at the max amount of hearts!`)
        }

        player.runCommandAsync(`scoreboard players add @s ls-death 1`)
        let health = player.getComponent(`health`) as EntityHealthComponent
        player.runCommandAsync(`clear @s lifesteal:heart 0 1`)
        player.runCommandAsync(`playsound random.orb @s`)
        lifestealLogs.push(`§6${time} > ${player.name} gained a heart`)
        if (health.current == getScore(player, `ls-death`) * 2 && health.current > 2) {
            system.runTimeout(() => {
                health.setCurrent(health.current - 2)
            }, 5)
        }
    }
})

/**
 * If true, you will lose hearts when dying from players AND non-player sources, if false, you will lose hearts ONLY when dying from a player
 * 
 * Can be toggled with ?envsteal <true/false>
 */
let envSteal: boolean
system.runInterval(() => {
    if (lifesteal.get(`envsteal`) == `true`) envSteal = true
    if (lifesteal.get(`envsteal`) == `false`) envSteal = false
}, 1)

/**
 * Lifesteal
 * 
 * -- The reason i'm not using EntityDie is for some reason it doesn't let me use data.deadEntity.target (If you damage someone then they die of fall damage, you gain a heart)
 */

world.events.entityHurt.subscribe(data => {
    let player = data.hurtEntity as Player
    let target = player.target as Player
    if (player.typeId !== `minecraft:player`) return
    let health = data.hurtEntity.getComponent(`health`) as EntityHealthComponent

    system.runTimeout(() => {
        if (envSteal == true) {
            if (!data.hurtEntity) return
            if (health.current <= 0) {
                lifestealLogs.push(`§6${time} > ${player.name} lost ${+lifesteal.get(`heartgl`)} heart(s)`)
                data.hurtEntity.runCommandAsync(`scoreboard players remove @s ls-death ${+lifesteal.get(`heartgl`)}`)

                if (!data.hurtEntity.target) return
                if (data.hurtEntity.target.typeId != `minecraft:player`) return

                if (health.current <= 0) data.hurtEntity.target.runCommandAsync(`scoreboard players add @s ls-death ${+lifesteal.get(`heartgl`)}`)
                lifestealLogs.push(`§6${time} > ${target.name} gained ${+lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`)
            }
        }
        if (envSteal == false) {
            if (!data.hurtEntity || !data.hurtEntity.target) return
            if (data.hurtEntity.target.typeId != `minecraft:player`) return
            if (health.current <= 0) {
                data.hurtEntity.target.runCommandAsync(`scoreboard players add @s ls-death ${+lifesteal.get(`heartgl`)}`)
                data.hurtEntity.runCommandAsync(`scoreboard players remove @s ls-death ${+lifesteal.get(`heartgl`)}`)
                lifestealLogs.push(`§6${time} > ${player.name} lost ${+lifesteal.get(`heartgl`)} heart(s)`)
                lifestealLogs.push(`§6${time} > ${target.name} gained ${+lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`)
            }
        }
    }, 1)
})

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let inv = player.getComponent(`inventory`) as EntityInventoryComponent
        for (let i = 0; i < inv.container.size; i++) {
            let it = inv.container.getItem(i)
            if (!it) continue
            if (it.typeId == `lifesteal:heart_of_life`) {
                if (it.getLore()[0] == `§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click or do ?revive§7 to use§r§7 ))`) continue
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_of_life`), it.amount)
                item.setLore([`§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click or do ?revive§7 to use§r§7 ))`])
                inv.container.setItem(i, item)
            }
            if (it.typeId == `lifesteal:heart_fragment`) {
                if (it.getLore()[0] == `§r§dUse 4 of these to craft a Heart!`) continue
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_fragment`), it.amount)
                item.setLore([`§r§dUse 4 of these to craft a Heart!`])
                inv.container.setItem(i, item)
            }
            if (it.typeId == `lifesteal:heart`) {
                if (it.getLore()[0] == `§r§dUse 4 of these to craft the Heart of Life\nor add to your heart count!`) continue
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart`), it.amount)
                item.setLore([`§r§dUse 4 of these to craft the Heart of Life\nor add to your heart count!`])
                inv.container.setItem(i, item)
            }
            if (it.typeId == `lifesteal:heart_essence`) {
                if (it.getLore()[0] == `§r§dUse this to craft a Heart!`) continue
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_essence`), it.amount)
                item.setLore([`§r§dUse this to craft a Heart!`])
                inv.container.setItem(i, item)
            }
        }
    }
}, 1)

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (player.hasTag(`ls-dead`) && !deadplayers.has(player.name)) {
            if (!revivelist.has(player.name)) deadplayers.set(`${player.name}`, `${player.name}`)
        }
        if (deadplayers.has(player.name) && !player.hasTag(`ls-dead`)) {
            deadplayers.delete(`${player.name}`)
        }
    }
}, 1)

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (revivelist.has(player.name)) {
            player.runCommandAsync(`tag @s add ls-revive`)
            revivelist.delete(player.name)
        }
    }
}, 1)

let banGhosts: Boolean
system.runInterval(() => {
    if (lifesteal.get(`banghosts`) == `true`) banGhosts = true
    if (lifesteal.get(`banghosts`) == `false`) banGhosts = false
}, 1)

system.runInterval(() => {
    if (banGhosts == false) return
    for (let player of world.getAllPlayers()) {
        if (!player.hasTag(`ls-dead`)) return
        player.runCommandAsync(`kick ${player.name} You are banned because you ran out of hearts!\n\n§bTo get revived, a player must use a Heart of Life item and revive you`)
    }
}, 40)

//Gui stuff

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (!player.hasTag(`welcome-gui_done`)) {
            player.addTag(`welcome-gui_done`)
            function form(player: Player) {
                const welcomeGui = new ActionFormData()
                    .title(`Welcome to lifesteal!`)
                    .body(`§gWelcome! This world has the §6Lifesteal Addon§g installed!\nThis lifesteal addon was created by §1§lGamerDos§r§g on §fYou§cTube§g, check him out!\n\n§lDo ${config.prefix}help to see all commands`)
                    .button(`Next`)
                welcomeGui.show(player).then(result => {
                    if (result.canceled && result.cancelationReason == `userBusy`) form(player)
                    if (result.canceled && result.cancelationReason == `userClosed`) form(player)
                    if (result.selection == 0) {
                        function nextForm(player: Player) {
                            const welcomeGui2 = new ActionFormData()
                                .title(`Lifesteal`)
                                .body(`§gYou are not allowed to repost this addon, if you plan on making a video on the addon, please link the MCPEDL page, you are not allowed to make your own download link, if you modify this addon, you must credit me\n\n§e- dsc.gg/gamerdos: §6GamerDos' Discord Server (Join if you need help setting the addon up or something)\n\n§e- mcpedl.com/lifesteal-addon: §6The MCPEDL page\n\n§e- ko-fi.com/gamerdos: §6Support what I do!`)
                                .button(`Accept and close`)
                            welcomeGui2.show(player).then(result => {
                                if (result.selection == 0) return
                                if (result.canceled && result.cancelationReason == `userClosed`) nextForm(player)
                            })
                        }
                        nextForm(player)
                    }
                })
            }
            form(player)
        }
    }
}, 1)

//Shows up when you die and become a ghost
world.events.playerSpawn.subscribe(({ player }) => {
    if (!player.hasTag(`ls-dead`)) return

    function ghostWelcomeForm() {
        let ghostWelcome = new ActionFormData()
            .title(`You died!`)
            .body(`§gYou have lost your last heart and have become a ghost!\n\n§bTo get revived, a player must use a Heart of Life item and revive you`)
            .button(`Alright`)
        ghostWelcome.show(player).then(result => {
            if (result.canceled && result.cancelationReason == `userBusy`) return ghostWelcomeForm
        })
    }
    ghostWelcomeForm()
})

world.events.beforeItemUse.subscribe(data => {
    if (data.item.typeId == `lifesteal:heart_of_life`) {
        let player = data.source as Player

        const options: string[] = []

        deadplayers.forEach(x => {
            options.push(x)
        })

        if (!options.length) {
            const errorGui = new ModalFormData()
                .title(`Revive GUI`)
                .dropdown(`Choose a player to revive`, [`No dead players!`])
            errorGui.show(player)
            return
        }

        const reviveGUI = new ModalFormData()
            .title(`Revive GUI`)
            .dropdown(`Choose a player to revive`, options)
        reviveGUI.show(player).then(result => {
            let formValues = result.formValues
            let target = options[formValues[0]]

            player.runCommandAsync(`clear @s lifesteal:heart_of_life 0 1`)
            revivelist.set(`${target}`, `${target}`)
            deadplayers.delete(`${target}`)
            player.runCommandAsync(`playsound heart_of_life_sound @a ~ ~ ~ 999`)
            sendAllMessage(player, `§b${player.name} has revived ${target}`)
        })
    }
    if (data.item.typeId == `lifesteal:withdraw_heart`) {
        let player = data.source as Player
        if (getScore(player, `ls-death`) == 1) return
        const withdrawGUI = new ModalFormData()
            .title(`Withdraw GUI`)
            .slider(`Choose how many hearts to withdraw`, 1, getScore(player, `ls-death`) - 1, 1)
        withdrawGUI.show(player).then(result => {
            let amount: any = result.formValues[0]

            player.runCommandAsync(`scoreboard players remove @s ls-death ${amount}`)
            player.runCommandAsync(`give @s lifesteal:heart ${amount}`)
            sendPlayerMessage(player, `§6Successfully withdrew hearts!`)
        })
    }
})
